@@ -37,6 +37,7 @@
 
 
 #define GET_WIDGET(x) (_gtk_builder_get_widget (data->builder, (x)))
+#define _OPEN_IN_BROWSER_RESPONSE 1
 
 
 enum {
@@ -62,6 +63,7 @@ typedef struct {
 	GList            *file_list;
 	GtkBuilder       *builder;
 	GtkWidget        *dialog;
+	GtkWidget        *list_view;
 	GtkWidget        *progress_dialog;
 	GList            *accounts;
 	PicasaWebUser    *user;
@@ -71,17 +73,20 @@ typedef struct {
 	GList            *albums;
 	GoogleConnection *conn;
 	PicasaWebService *picasaweb;
+	PicasaWebAlbum   *album;
 	GCancellable     *cancellable;
 } DialogData;
 
 
 static void
-export_dialog_destroy_cb (GtkWidget  *widget,
-			  DialogData *data)
+destroy_dialog (DialogData *data)
 {
+	if (data->dialog != NULL)
+		gtk_widget_destroy (data->dialog);
 	if (data->conn != NULL)
 		gth_task_completed (GTH_TASK (data->conn), NULL);
 	_g_object_unref (data->cancellable);
+	_g_object_unref (data->album);
 	_g_object_unref (data->picasaweb);
 	_g_object_unref (data->conn);
 	_g_object_list_unref (data->albums);
@@ -97,7 +102,73 @@ export_dialog_destroy_cb (GtkWidget  *widget,
 }
 
 
-static void get_album_list (DialogData *data);
+static void
+completed_messagedialog_response_cb (GtkDialog *dialog,
+				     int        response_id,
+				     gpointer   user_data)
+{
+	DialogData *data = user_data;
+
+	switch (response_id) {
+	case GTK_RESPONSE_DELETE_EVENT:
+	case GTK_RESPONSE_CLOSE:
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_DELETE_EVENT);
+		break;
+
+	case _OPEN_IN_BROWSER_RESPONSE:
+		{
+			char   *url = NULL;
+			GError *error = NULL;
+
+			gtk_widget_destroy (GTK_WIDGET (dialog));
+
+			if (data->album != NULL) {
+				if (data->album->alternate_url != NULL)
+					url = g_strdup (data->album->alternate_url);
+				else
+					url = g_strconcat ("http://picasaweb.google.com/", data->user->id, "/", data->album->id, NULL);
+			}
+			else
+				url = g_strconcat ("http://picasaweb.google.com/", data->user->id, NULL);
+
+			if ((url != NULL) && ! gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (dialog)), url, 0, &error)) {
+				if (data->conn != NULL)
+					gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
+				_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->browser), _("Could not connect to the server"), &error);
+			}
+			gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_DELETE_EVENT);
+
+			g_free (url);
+		}
+		break;
+
+	default:
+		break;
+	}
+}
+
+
+static void
+export_completed_with_success (DialogData *data)
+{
+	GtkBuilder *builder;
+	GtkWidget  *dialog;
+
+	gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
+
+	builder = _gtk_builder_new_from_file ("picasa-web-export-completed.ui", "picasaweb");
+	dialog = _gtk_builder_get_widget (builder, "completed_messagedialog");
+	g_object_set_data_full (G_OBJECT (dialog), "builder", builder, g_object_unref);
+	g_signal_connect (dialog,
+			  "response",
+			  G_CALLBACK (completed_messagedialog_response_cb),
+			  data);
+
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (data->browser));
+	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+	gtk_window_present (GTK_WINDOW (dialog));
+}
 
 
 static void
@@ -109,16 +180,14 @@ post_photos_ready_cb (GObject      *source_object,
 	PicasaWebService *picasaweb = PICASA_WEB_SERVICE (source_object);
 	GError           *error = NULL;
 
-	gth_task_dialog (GTH_TASK (data->conn), TRUE);
-
 	if (! picasa_web_service_post_photos_finish (picasaweb, result, &error)) {
 		if (data->conn != NULL)
-			gth_task_dialog (GTH_TASK (data->conn), TRUE);
+			gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not upload the files"), &error);
 		return;
 	}
 
-	get_album_list (data);
+	export_completed_with_success (data);
 }
 
 
@@ -137,37 +206,37 @@ export_dialog_response_cb (GtkDialog *dialog,
 	case GTK_RESPONSE_DELETE_EVENT:
 	case GTK_RESPONSE_CANCEL:
 		picasa_web_accounts_save_to_file (data->accounts, data->email);
-		gtk_widget_destroy (data->dialog);
+		gth_file_list_cancel (GTH_FILE_LIST (data->list_view), (DataFunc) destroy_dialog, data);
 		break;
 
 	case GTK_RESPONSE_OK:
 		{
-			GtkTreeModel   *tree_model;
-			GtkTreeIter     iter;
-			PicasaWebAlbum *album;
-			GList          *file_list;
+			GtkTreeModel *tree_model;
+			GtkTreeIter   iter;
+			GList        *file_list;
 
 			if (! gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (GET_WIDGET ("albums_treeview"))), &tree_model, &iter)) {
 				gtk_widget_set_sensitive (GET_WIDGET ("upload_button"), FALSE);
 				return;
 			}
 
+			_g_clear_object (&data->album);
 			gtk_tree_model_get (tree_model, &iter,
-					    ALBUM_DATA_COLUMN, &album,
+					    ALBUM_DATA_COLUMN, &data->album,
 					    -1);
 
-			gth_task_dialog (GTH_TASK (data->conn), FALSE);
+			gtk_widget_hide (data->dialog);
+			gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
 
 			file_list = gth_file_data_list_to_file_list (data->file_list);
 			picasa_web_service_post_photos (data->picasaweb,
-							album,
+							data->album,
 							file_list,
 							data->cancellable,
 							post_photos_ready_cb,
 							data);
 
 			_g_object_list_unref (file_list);
-			g_object_unref (album);
 		}
 		break;
 
@@ -253,7 +322,7 @@ show_export_dialog (DialogData *data)
 {
 	update_account_list (data);
 	update_album_list (data);
-	gth_task_dialog (GTH_TASK (data->conn), TRUE);
+	gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (data->browser));
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
@@ -274,9 +343,9 @@ list_albums_ready_cb (GObject      *source_object,
 	data->albums = picasa_web_service_list_albums_finish (picasaweb, result, &error);
 	if (error != NULL) {
 		if (data->conn != NULL)
-			gth_task_dialog (GTH_TASK (data->conn), TRUE);
+			gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not get the album list"), &error);
-		gtk_widget_destroy (data->dialog);
+		gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_DELETE_EVENT);
 		return;
 	}
 
@@ -332,9 +401,9 @@ connection_ready_cb (GObject      *source_object,
 		}
 		else {
 			if (data->conn != NULL)
-				gth_task_dialog (GTH_TASK (data->conn), TRUE);
+				gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 			_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not connect to the server"), &error);
-			gtk_widget_destroy (data->dialog);
+			gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_DELETE_EVENT);
 		}
 		return;
 	}
@@ -381,7 +450,7 @@ account_properties_dialog_response_cb (GtkDialog *dialog,
 	case GTK_RESPONSE_DELETE_EVENT:
 	case GTK_RESPONSE_CANCEL:
 		gtk_widget_destroy (GTK_WIDGET (dialog));
-		gtk_widget_destroy (data->dialog);
+		gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_DELETE_EVENT);
 		break;
 
 	case GTK_RESPONSE_OK:
@@ -408,7 +477,7 @@ account_properties_dialog (DialogData *data,
 	GtkWidget *dialog;
 
 	if (data->conn != NULL)
-		gth_task_dialog (GTH_TASK (data->conn), TRUE);
+		gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 
 	dialog = picasa_account_properties_dialog_new (email, NULL, NULL);
 	g_signal_connect (dialog,
@@ -427,11 +496,11 @@ static void
 connect_to_server_step2 (DialogData *data)
 {
 	if (data->password == NULL) {
-		gth_task_dialog (GTH_TASK (data->conn), TRUE);
+		gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 		account_properties_dialog (data, data->email);
 	}
 	else {
-		gth_task_dialog (GTH_TASK (data->conn), FALSE);
+		gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
 		google_connection_connect (data->conn,
 					   data->email,
 					   data->password,
@@ -500,7 +569,7 @@ challange_account_dialog_response_cb (GtkDialog *dialog,
 	case GTK_RESPONSE_DELETE_EVENT:
 	case GTK_RESPONSE_CANCEL:
 		gtk_widget_destroy (GTK_WIDGET (dialog));
-		gtk_widget_destroy (data->dialog);
+		gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_DELETE_EVENT);
 		break;
 
 	case GTK_RESPONSE_OK:
@@ -547,7 +616,7 @@ account_chooser_dialog_response_cb (GtkDialog *dialog,
 	case GTK_RESPONSE_DELETE_EVENT:
 	case GTK_RESPONSE_CANCEL:
 		gtk_widget_destroy (GTK_WIDGET (dialog));
-		gtk_widget_destroy (data->dialog);
+		gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_DELETE_EVENT);
 		break;
 
 	case GTK_RESPONSE_OK:
@@ -588,7 +657,7 @@ create_album_ready_cb (GObject      *source_object,
 	album = picasa_web_service_create_album_finish (picasaweb, result, &error);
 	if (error != NULL) {
 		if (data->conn != NULL)
-			gth_task_dialog (GTH_TASK (data->conn), TRUE);
+			gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not create the album"), &error);
 		return;
 	}
@@ -662,7 +731,7 @@ auto_select_account (DialogData *data)
 {
 	gtk_widget_hide (data->dialog);
 	if (data->conn != NULL)
-		gth_task_dialog (GTH_TASK (data->conn), FALSE);
+		gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
 
 	if (data->accounts != NULL) {
 		if (data->email != NULL) {
@@ -675,7 +744,7 @@ auto_select_account (DialogData *data)
 		else {
 			GtkWidget *dialog;
 
-			gth_task_dialog (GTH_TASK (data->conn), TRUE);
+			gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 			dialog = picasa_account_chooser_dialog_new (data->accounts, data->email);
 			g_signal_connect (dialog,
 					  "response",
@@ -789,7 +858,6 @@ dlg_export_to_picasaweb (GthBrowser *browser,
 		         GList      *file_list)
 {
 	DialogData       *data;
-	GtkWidget        *list_view;
 	GtkTreeSelection *selection;
 	GList            *scan;
 	int               n_total;
@@ -829,6 +897,8 @@ dlg_export_to_picasaweb (GthBrowser *browser,
 						NULL);
 	}
 
+	_gtk_window_resize_to_fit_screen_height (data->dialog, 500);
+
 	data->file_list = NULL;
 	n_total = 0;
 	total_size = 0;
@@ -847,16 +917,17 @@ dlg_export_to_picasaweb (GthBrowser *browser,
 			data->file_list = g_list_prepend (data->file_list, g_object_ref (file_data));
 		}
 	}
+	data->file_list = g_list_reverse (data->file_list);
 
 	if (data->file_list == NULL) {
 		GError *error;
 
 		if (data->conn != NULL)
-			gth_task_dialog (GTH_TASK (data->conn), TRUE);
+			gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 
 		error = g_error_new_literal (GTH_ERROR, GTH_ERROR_GENERIC, _("No valid file selected."));
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (browser), _("Could not export the files"), &error);
-		export_dialog_destroy_cb (NULL, data);
+		destroy_dialog (data);
 		return;
 	}
 
@@ -868,15 +939,15 @@ dlg_export_to_picasaweb (GthBrowser *browser,
 
 	/* Set the widget data */
 
-	list_view = gth_file_list_new (GTH_FILE_LIST_TYPE_NO_SELECTION);
-	gth_file_list_set_thumb_size (GTH_FILE_LIST (list_view), 112);
-	gth_file_view_set_spacing (GTH_FILE_VIEW (gth_file_list_get_view (GTH_FILE_LIST (list_view))), 0);
-	gth_file_list_enable_thumbs (GTH_FILE_LIST (list_view), TRUE);
-	gth_file_list_set_caption (GTH_FILE_LIST (list_view), "none");
-	gth_file_list_set_sort_func (GTH_FILE_LIST (list_view), gth_main_get_sort_type ("file::name")->cmp_func, FALSE);
-	gtk_widget_show (list_view);
-	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("images_box")), list_view, TRUE, TRUE, 0);
-	gth_file_list_set_files (GTH_FILE_LIST (list_view), data->file_list);
+	data->list_view = gth_file_list_new (GTH_FILE_LIST_TYPE_NO_SELECTION, FALSE);
+	gth_file_list_set_thumb_size (GTH_FILE_LIST (data->list_view), 112);
+	gth_file_view_set_spacing (GTH_FILE_VIEW (gth_file_list_get_view (GTH_FILE_LIST (data->list_view))), 0);
+	gth_file_list_enable_thumbs (GTH_FILE_LIST (data->list_view), TRUE);
+	gth_file_list_set_caption (GTH_FILE_LIST (data->list_view), "none");
+	gth_file_list_set_sort_func (GTH_FILE_LIST (data->list_view), gth_main_get_sort_type ("file::name")->cmp_func, FALSE);
+	gtk_widget_show (data->list_view);
+	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("images_box")), data->list_view, TRUE, TRUE, 0);
+	gth_file_list_set_files (GTH_FILE_LIST (data->list_view), data->file_list);
 
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (GET_WIDGET ("album_liststore")), ALBUM_NAME_COLUMN, GTK_SORT_ASCENDING);
 
@@ -884,10 +955,10 @@ dlg_export_to_picasaweb (GthBrowser *browser,
 
 	/* Set the signals handlers. */
 
-	g_signal_connect (G_OBJECT (data->dialog),
-			  "destroy",
-			  G_CALLBACK (export_dialog_destroy_cb),
-			  data);
+	g_signal_connect (data->dialog,
+			  "delete-event",
+			  G_CALLBACK (gtk_widget_hide_on_delete),
+			  NULL);
 	g_signal_connect (data->dialog,
 			  "response",
 			  G_CALLBACK (export_dialog_response_cb),
