@@ -22,12 +22,15 @@
 
 #include <config.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
+#ifdef HAVE_CLUTTER
 #include <clutter/clutter.h>
 #include <clutter-gtk/clutter-gtk.h>
+#endif /* HAVE_CLUTTER */
 #if HAVE_GSTREAMER
 #include <gst/gst.h>
 #include <extensions/gstreamer_utils/gstreamer-utils.h>
-#endif
+#endif /* HAVE_GSTREAMER */
 #include "gth-slideshow.h"
 #include "gth-transition.h"
 
@@ -35,33 +38,44 @@
 #define DEFAULT_DELAY 2000
 
 
+typedef enum {
+	GTH_SLIDESHOW_DIRECTION_FORWARD,
+	GTH_SLIDESHOW_DIRECTION_BACKWARD
+} GthSlideshowDirection;
+
+
 struct _GthSlideshowPrivate {
-	GthBrowser        *browser;
-	GList             *file_list; /* GthFileData */
-	gboolean           automatic;
-	gboolean           wrap_around;
-	GList             *current;
-	GthImagePreloader *preloader;
-	GList             *transitions; /* GthTransition */
-	int                n_transitions;
-	GthTransition     *transition;
-	ClutterTimeline   *timeline;
-	ClutterActor      *image1;
-	ClutterActor      *image2;
-	guint              next_event;
-	guint              delay;
-	guint              hide_cursor_event;
-	GRand             *rand;
-	gboolean           first_show;
-	gboolean           one_loaded;
-	char             **audio_files;
-	gboolean           audio_loop;
-#if HAVE_GSTREAMER
-	int                current_audio_file;
-	GstElement        *playbin;
+	GthProjector          *projector;
+	GthBrowser            *browser;
+	GList                 *file_list; /* GthFileData */
+	gboolean               automatic;
+	gboolean               wrap_around;
+	GList                 *current;
+	GthImagePreloader     *preloader;
+	GList                 *transitions; /* GthTransition */
+	int                    n_transitions;
+	GthTransition         *transition;
+	GthSlideshowDirection  direction;
+#if HAVE_CLUTTER
+	ClutterTimeline       *timeline;
+	ClutterActor          *image1;
+	ClutterActor          *image2;
 #endif
-	gboolean           paused;
-	gboolean           animating;
+	GtkWidget             *viewer;
+	guint                  next_event;
+	guint                  delay;
+	guint                  hide_cursor_event;
+	GRand                 *rand;
+	gboolean               first_show;
+	gboolean               one_loaded;
+	char                 **audio_files;
+	gboolean               audio_loop;
+#if HAVE_GSTREAMER
+	int                    current_audio_file;
+	GstElement            *playbin;
+#endif
+	gboolean               paused;
+	gboolean               animating;
 };
 
 
@@ -101,7 +115,7 @@ _gth_slideshow_load_current_image (GthSlideshow *self)
 			return;
 		}
 
-		if (clutter_timeline_get_direction (self->priv->timeline) == CLUTTER_TIMELINE_FORWARD)
+		if (self->priv->direction == GTH_SLIDESHOW_DIRECTION_FORWARD)
 			self->priv->current = g_list_first (self->priv->file_list);
 		else
 			self->priv->current = g_list_last (self->priv->file_list);
@@ -123,6 +137,486 @@ _gth_slideshow_load_current_image (GthSlideshow *self)
 }
 
 
+static void
+_gth_slideshow_load_next_image (GthSlideshow *self)
+{
+	self->priv->projector->load_next_image (self); /* FIXME */
+	self->priv->direction = GTH_SLIDESHOW_DIRECTION_FORWARD;
+
+	if (self->priv->paused)
+		return;
+
+	self->priv->current = self->priv->current->next;
+	_gth_slideshow_load_current_image (self);
+}
+
+
+static void
+_gth_slideshow_load_prev_image (GthSlideshow *self)
+{
+	self->priv->projector->load_prev_image (self); /* FIXME */
+	self->priv->direction = GTH_SLIDESHOW_DIRECTION_BACKWARD;
+
+	if (self->priv->paused)
+		return;
+
+	self->priv->current = self->priv->current->prev;
+	_gth_slideshow_load_current_image (self);
+}
+
+
+static gboolean
+next_image_cb (gpointer user_data)
+{
+	GthSlideshow *self = user_data;
+
+	if (self->priv->next_event != 0) {
+		g_source_remove (self->priv->next_event);
+		self->priv->next_event = 0;
+	}
+	_gth_slideshow_load_next_image (self);
+
+	return FALSE;
+}
+
+
+static void
+view_next_image_automatically (GthSlideshow *self)
+{
+	if (self->priv->automatic) {
+		if (self->priv->next_event != 0)
+			g_source_remove (self->priv->next_event);
+		self->priv->next_event = g_timeout_add (self->priv->delay, next_image_cb, self);
+	}
+}
+
+
+static void
+image_preloader_requested_ready_cb (GthImagePreloader *preloader,
+				    GError            *error,
+				    gpointer           user_data)
+{
+	GthSlideshow   *self = user_data;
+	GthImageLoader *image_loader;
+	GdkPixbuf      *pixbuf;
+
+	if (error != NULL) {
+		g_clear_error (&error);
+		_gth_slideshow_load_next_image (self);
+		return;
+	}
+
+	image_loader = gth_image_preloader_get_loader (self->priv->preloader, (GthFileData *) self->priv->current->data);
+	if (image_loader == NULL) {
+		_gth_slideshow_load_next_image (self);
+		return;
+	}
+
+	pixbuf = gth_image_loader_get_pixbuf (image_loader);
+	if (pixbuf == NULL) {
+		_gth_slideshow_load_next_image (self);
+		return;
+	}
+
+	self->priv->one_loaded = TRUE;
+	self->priv->projector->image_ready (self, pixbuf); /* FIXME */
+}
+
+
+static void
+gth_slideshow_init (GthSlideshow *self)
+{
+	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_SLIDESHOW, GthSlideshowPrivate);
+	self->priv->file_list = NULL;
+	self->priv->next_event = 0;
+	self->priv->delay = DEFAULT_DELAY;
+	self->priv->automatic = FALSE;
+	self->priv->wrap_around = FALSE;
+	self->priv->transitions = NULL;
+	self->priv->n_transitions = 0;
+	self->priv->rand = g_rand_new ();
+	self->priv->first_show = TRUE;
+	self->priv->audio_files = NULL;
+	self->priv->paused = FALSE;
+	self->priv->animating = FALSE;
+	self->priv->direction = GTH_SLIDESHOW_DIRECTION_FORWARD;
+
+	self->priv->preloader = gth_image_preloader_new ();
+	g_signal_connect (self->priv->preloader,
+			  "requested_ready",
+			  G_CALLBACK (image_preloader_requested_ready_cb),
+			  self);
+}
+
+
+static void
+gth_slideshow_finalize (GObject *object)
+{
+	GthSlideshow *self = GTH_SLIDESHOW (object);
+
+	if (self->priv->next_event != 0)
+		g_source_remove (self->priv->next_event);
+	if (self->priv->hide_cursor_event != 0)
+		g_source_remove (self->priv->hide_cursor_event);
+
+	_g_object_list_unref (self->priv->file_list);
+	_g_object_unref (self->priv->browser);
+	_g_object_unref (self->priv->preloader);
+	_g_object_list_unref (self->priv->transitions);
+	g_rand_free (self->priv->rand);
+	g_strfreev (self->priv->audio_files);
+	self->priv->projector->finalize (self); /* FIXME */
+
+#if HAVE_GSTREAMER
+	if (self->priv->playbin != NULL) {
+		gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
+		gst_object_unref (GST_OBJECT (self->priv->playbin));
+		self->priv->playbin = NULL;
+	}
+#endif
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+
+static void
+gth_slideshow_class_init (GthSlideshowClass *klass)
+{
+	GObjectClass *gobject_class;
+
+	parent_class = g_type_class_peek_parent (klass);
+	g_type_class_add_private (klass, sizeof (GthSlideshowPrivate));
+
+	gobject_class = G_OBJECT_CLASS (klass);
+	gobject_class->finalize = gth_slideshow_finalize;
+}
+
+
+GType
+gth_slideshow_get_type (void)
+{
+	static GType type = 0;
+
+	if (! type) {
+		GTypeInfo type_info = {
+			sizeof (GthSlideshowClass),
+			NULL,
+			NULL,
+			(GClassInitFunc) gth_slideshow_class_init,
+			NULL,
+			NULL,
+			sizeof (GthSlideshow),
+			0,
+			(GInstanceInitFunc) gth_slideshow_init
+		};
+
+		type = g_type_register_static (GTK_TYPE_WINDOW,
+					       "GthSlideshow",
+					       &type_info,
+					       0);
+	}
+
+	return type;
+}
+
+
+static gboolean
+hide_cursor_cb (gpointer data)
+{
+	GthSlideshow *self = data;
+
+	g_source_remove (self->priv->hide_cursor_event);
+	self->priv->hide_cursor_event = 0;
+	self->priv->projector->hide_cursor (self); /* FIXME */
+
+	return FALSE;
+}
+
+
+static void
+_gth_slideshow_toggle_pause (GthSlideshow *self)
+{
+	self->priv->paused = ! self->priv->paused;
+	if (self->priv->paused) {
+		self->priv->projector->paused (self); /* FIXME */
+#if HAVE_GSTREAMER
+		if (self->priv->playbin != NULL)
+			gst_element_set_state (self->priv->playbin, GST_STATE_PAUSED);
+#endif
+	}
+	else { /* resume */
+		_gth_slideshow_load_next_image (self);
+#if HAVE_GSTREAMER
+		if (self->priv->playbin != NULL)
+			gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
+#endif
+	}
+}
+
+
+#if HAVE_GSTREAMER
+static void
+bus_message_cb (GstBus     *bus,
+                GstMessage *message,
+                gpointer    user_data)
+{
+	GthSlideshow *self = user_data;
+
+	if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_EOS) {
+		self->priv->current_audio_file++;
+		if ((self->priv->audio_files[self->priv->current_audio_file] == NULL)
+		    && self->priv->audio_loop)
+		{
+			self->priv->current_audio_file = 0;
+		}
+		gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
+		g_object_set (G_OBJECT (self->priv->playbin), "uri", self->priv->audio_files[self->priv->current_audio_file], NULL);
+		gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
+	}
+}
+#endif
+
+
+static void
+gth_slideshow_show_cb (GtkWidget    *widget,
+		       GthSlideshow *self)
+{
+	if (! self->priv->first_show)
+		return;
+
+#if HAVE_GSTREAMER
+	if (gstreamer_init ()) {
+		if ((self->priv->audio_files != NULL) && (self->priv->audio_files[0] != NULL)) {
+			self->priv->current_audio_file = 0;
+			if (self->priv->playbin == NULL) {
+				GstBus *bus;
+
+				self->priv->playbin = gst_element_factory_make ("playbin", "playbin");
+				bus = gst_pipeline_get_bus (GST_PIPELINE (self->priv->playbin));
+				gst_bus_add_signal_watch (bus);
+				g_signal_connect (bus, "message", G_CALLBACK (bus_message_cb), self);
+			}
+			else
+				gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
+			g_object_set (G_OBJECT (self->priv->playbin), "uri", self->priv->audio_files[self->priv->current_audio_file], NULL);
+			gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
+		}
+	}
+#endif
+
+	_gth_slideshow_load_current_image (self);
+	self->priv->first_show = FALSE;
+}
+
+
+static void
+_gth_slideshow_construct (GthSlideshow *self,
+			  GthProjector *projector,
+			  GthBrowser   *browser,
+			  GList        *file_list)
+{
+	self->priv->projector = projector;
+	self->priv->browser = _g_object_ref (browser);
+	self->priv->file_list = _g_object_list_ref (file_list);
+	self->priv->current = self->priv->file_list;
+	self->priv->one_loaded = FALSE;
+
+	self->priv->projector->construct (self); /* FIXME */
+
+	g_signal_connect (self, "show", G_CALLBACK (gth_slideshow_show_cb), self);
+}
+
+
+GtkWidget *
+gth_slideshow_new (GthProjector *projector,
+		   GthBrowser   *browser,
+		   GList        *file_list /* GthFileData */)
+{
+	GthSlideshow *window;
+
+	g_return_val_if_fail (projector != NULL, NULL);
+
+	window = (GthSlideshow *) g_object_new (GTH_TYPE_SLIDESHOW, NULL);
+	_gth_slideshow_construct (window, projector, browser, file_list);
+
+	return (GtkWidget*) window;
+}
+
+
+void
+gth_slideshow_set_delay (GthSlideshow *self,
+			 guint         msecs)
+{
+	self->priv->delay = msecs;
+}
+
+
+void
+gth_slideshow_set_automatic (GthSlideshow *self,
+			     gboolean      automatic)
+{
+	self->priv->automatic = automatic;
+}
+
+
+void
+gth_slideshow_set_wrap_around (GthSlideshow *self,
+			       gboolean      wrap_around)
+{
+	self->priv->wrap_around = wrap_around;
+}
+
+
+void
+gth_slideshow_set_transitions (GthSlideshow *self,
+			       GList        *transitions)
+{
+	_g_object_list_unref (self->priv->transitions);
+	self->priv->transitions = _g_object_list_ref (transitions);
+	self->priv->n_transitions = g_list_length (self->priv->transitions);
+}
+
+
+void
+gth_slideshow_set_playlist (GthSlideshow  *self,
+			    char         **files)
+{
+	self->priv->audio_files = g_strdupv (files);
+	self->priv->audio_loop = TRUE;
+}
+
+
+/* -- default projector -- */
+
+
+static void
+default_projector_load_next_image (GthSlideshow *self)
+{
+	/* void */
+}
+
+
+static void
+default_projector_load_prev_image (GthSlideshow *self)
+{
+	/* void */
+}
+
+
+static void
+default_projector_image_ready (GthSlideshow *self,
+			       GdkPixbuf    *pixbuf)
+{
+	gth_image_viewer_set_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer), pixbuf);
+	view_next_image_automatically (self);
+}
+
+
+static void
+default_projector_finalize (GthSlideshow *self)
+{
+	/* void */
+}
+
+
+static void
+default_projector_hide_cursor (GthSlideshow *self)
+{
+	gth_image_viewer_hide_cursor (GTH_IMAGE_VIEWER (self->priv->viewer));
+}
+
+
+static void
+default_projector_paused (GthSlideshow *self)
+{
+	/* void */
+}
+
+
+static void
+viewer_event_cb (GtkWidget    *widget,
+	         GdkEvent     *event,
+	         GthSlideshow *self)
+{
+	if (event->type == GDK_MOTION_NOTIFY) {
+		gth_image_viewer_show_cursor (GTH_IMAGE_VIEWER (self->priv->viewer));
+		if (self->priv->hide_cursor_event != 0)
+			g_source_remove (self->priv->hide_cursor_event);
+		self->priv->hide_cursor_event = g_timeout_add (HIDE_CURSOR_DELAY, hide_cursor_cb, self);
+	}
+	else if (event->type == GDK_BUTTON_PRESS) {
+		switch (((GdkEventButton *) event)->button) {
+		case 1:
+			_gth_slideshow_load_next_image (self);
+			break;
+		case 3:
+			_gth_slideshow_load_prev_image (self);
+			break;
+		default:
+			break;
+		}
+	}
+	else if (event->type == GDK_KEY_RELEASE) {
+		switch (((GdkEventKey *) event)->keyval) {
+		case GDK_Escape:
+			_gth_slideshow_close (self);
+			break;
+
+		case GDK_space:
+			_gth_slideshow_toggle_pause (self);
+			break;
+
+		case GDK_Up:
+		case GDK_Right:
+			_gth_slideshow_load_next_image (self);
+			break;
+
+		case GDK_Down:
+		case GDK_Left:
+			_gth_slideshow_load_prev_image (self);
+			break;
+		}
+	}
+}
+
+
+static void
+default_projector_construct (GthSlideshow *self)
+{
+	self->priv->viewer = gth_image_viewer_new ();
+	gth_image_viewer_set_black_background (GTH_IMAGE_VIEWER (self->priv->viewer), TRUE);
+	gth_image_viewer_hide_frame (GTH_IMAGE_VIEWER (self->priv->viewer));
+	gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_FIT_SIZE_IF_LARGER);
+	gth_image_viewer_set_zoom_change (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_ZOOM_CHANGE_FIT_SIZE_IF_LARGER);
+	gth_image_viewer_set_transp_type (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_TRANSP_TYPE_BLACK);
+	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_ZOOM_QUALITY_LOW);
+
+	g_signal_connect (self->priv->viewer, "button-press-event", G_CALLBACK (viewer_event_cb), self);
+	g_signal_connect (self->priv->viewer, "motion-notify-event", G_CALLBACK (viewer_event_cb), self);
+	g_signal_connect (self->priv->viewer, "key-release-event", G_CALLBACK (viewer_event_cb), self);
+
+	gtk_widget_show (self->priv->viewer);
+	gtk_container_add (GTK_CONTAINER (self), self->priv->viewer);
+}
+
+
+GthProjector default_projector = {
+	default_projector_construct,
+	default_projector_paused,
+	default_projector_hide_cursor,
+	default_projector_finalize,
+	default_projector_image_ready,
+	default_projector_load_prev_image,
+	default_projector_load_next_image
+};
+
+
+#ifdef HAVE_CLUTTER
+
+
+/* -- clutter projector -- */
+
+
 static void
 _gth_slideshow_swap_current_and_next (GthSlideshow *self)
 {
@@ -212,48 +706,24 @@ _gth_slideshow_animation_completed (GthSlideshow *self)
 
 
 static void
-_gth_slideshow_load_next_image (GthSlideshow *self)
+clutter_projector_load_next_image (GthSlideshow *self)
 {
 	if (clutter_timeline_is_playing (self->priv->timeline)) {
 		clutter_timeline_pause (self->priv->timeline);
 		_gth_slideshow_animation_completed (self);
 	}
-
-	if (self->priv->paused)
-		return;
-
-	self->priv->current = self->priv->current->next;
 	clutter_timeline_set_direction (self->priv->timeline, CLUTTER_TIMELINE_FORWARD);
-	_gth_slideshow_load_current_image (self);
 }
 
 
 static void
-_gth_slideshow_load_prev_image (GthSlideshow *self)
+clutter_projector_load_prev_image (GthSlideshow *self)
 {
 	if (clutter_timeline_is_playing (self->priv->timeline)) {
 		clutter_timeline_pause (self->priv->timeline);
 		_gth_slideshow_animation_completed (self);
 	}
-
-	self->priv->current = self->priv->current->prev;
 	clutter_timeline_set_direction (self->priv->timeline, CLUTTER_TIMELINE_BACKWARD);
-	_gth_slideshow_load_current_image (self);
-}
-
-
-static gboolean
-next_image_cb (gpointer user_data)
-{
-	GthSlideshow *self = user_data;
-
-	if (self->priv->next_event != 0) {
-		g_source_remove (self->priv->next_event);
-		self->priv->next_event = 0;
-	}
-	_gth_slideshow_load_next_image (self);
-
-	return FALSE;
 }
 
 
@@ -262,12 +732,7 @@ animation_completed_cb (ClutterTimeline *timeline,
 			GthSlideshow    *self)
 {
 	_gth_slideshow_animation_completed (self);
-
-	if (self->priv->automatic) {
-		if (self->priv->next_event != 0)
-			g_source_remove (self->priv->next_event);
-		self->priv->next_event = g_timeout_add (self->priv->delay, next_image_cb, self);
-	}
+	view_next_image_automatically (self);
 }
 
 
@@ -306,43 +771,19 @@ _gth_slideshow_get_transition (GthSlideshow *self)
 
 
 static void
-image_preloader_requested_ready_cb (GthImagePreloader *preloader,
-				    GError            *error,
-				    gpointer           user_data)
+clutter_projector_image_ready (GthSlideshow *self,
+			       GdkPixbuf    *pixbuf)
 {
-	GthSlideshow   *self = user_data;
-	GthImageLoader *image_loader;
-	GdkPixbuf      *pixbuf;
-	GdkPixbuf      *image;
-	ClutterActor   *texture;
-	int             pixbuf_w, pixbuf_h;
-	float           stage_w, stage_h;
-	int             pixbuf_x, pixbuf_y;
+	GdkPixbuf    *image;
+	ClutterActor *texture;
+	int           pixbuf_w, pixbuf_h;
+	float         stage_w, stage_h;
+	int           pixbuf_x, pixbuf_y;
 
-	if (error != NULL) {
-		g_clear_error (&error);
-		_gth_slideshow_load_next_image (self);
-		return;
-	}
-
-	self->priv->one_loaded = TRUE;
 	clutter_actor_get_size (self->stage, &stage_w, &stage_h);
-
 	if ((stage_w == 0) || (stage_h == 0))
 		return;
 
-	image_loader = gth_image_preloader_get_loader (self->priv->preloader, (GthFileData *) self->priv->current->data);
-	if (image_loader == NULL) {
-		_gth_slideshow_load_next_image (self);
-		return;
-	}
-
-	pixbuf = gth_image_loader_get_pixbuf (image_loader);
-	if (image_loader == NULL) {
-		_gth_slideshow_load_next_image (self);
-		return;
-	}
-
 	image = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (pixbuf),
 				FALSE,
 				gdk_pixbuf_get_bits_per_sample (pixbuf),
@@ -393,135 +834,25 @@ image_preloader_requested_ready_cb (GthImagePreloader *preloader,
 
 
 static void
-gth_slideshow_init (GthSlideshow *self)
+clutter_projector_finalize (GthSlideshow *self)
 {
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_SLIDESHOW, GthSlideshowPrivate);
-	self->priv->file_list = NULL;
-	self->priv->next_event = 0;
-	self->priv->delay = DEFAULT_DELAY;
-	self->priv->automatic = FALSE;
-	self->priv->wrap_around = FALSE;
-	self->priv->transitions = NULL;
-	self->priv->n_transitions = 0;
-	self->priv->rand = g_rand_new ();
-	self->priv->first_show = TRUE;
-	self->priv->audio_files = NULL;
-	self->priv->paused = FALSE;
-	self->priv->animating = FALSE;
-
-	self->priv->preloader = gth_image_preloader_new ();
-	g_signal_connect (self->priv->preloader,
-			  "requested_ready",
-			  G_CALLBACK (image_preloader_requested_ready_cb),
-			  self);
-}
-
-
-static void
-gth_slideshow_finalize (GObject *object)
-{
-	GthSlideshow *self = GTH_SLIDESHOW (object);
-
-	if (self->priv->next_event != 0)
-		g_source_remove (self->priv->next_event);
-	if (self->priv->hide_cursor_event != 0)
-		g_source_remove (self->priv->hide_cursor_event);
-
-	_g_object_list_unref (self->priv->file_list);
-	_g_object_unref (self->priv->browser);
-	_g_object_unref (self->priv->preloader);
 	_g_object_unref (self->priv->timeline);
-	_g_object_list_unref (self->priv->transitions);
-	g_rand_free (self->priv->rand);
-	g_strfreev (self->priv->audio_files);
-
-#if HAVE_GSTREAMER
-	if (self->priv->playbin != NULL) {
-		gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
-		gst_object_unref (GST_OBJECT (self->priv->playbin));
-		self->priv->playbin = NULL;
-	}
-#endif
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 
 static void
-gth_slideshow_class_init (GthSlideshowClass *klass)
+clutter_projector_hide_cursor (GthSlideshow *self)
 {
-	GObjectClass *gobject_class;
-
-	parent_class = g_type_class_peek_parent (klass);
-	g_type_class_add_private (klass, sizeof (GthSlideshowPrivate));
-
-	gobject_class = G_OBJECT_CLASS (klass);
-	gobject_class->finalize = gth_slideshow_finalize;
-}
-
-
-GType
-gth_slideshow_get_type (void)
-{
-	static GType type = 0;
-
-	if (! type) {
-		GTypeInfo type_info = {
-			sizeof (GthSlideshowClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) gth_slideshow_class_init,
-			NULL,
-			NULL,
-			sizeof (GthSlideshow),
-			0,
-			(GInstanceInitFunc) gth_slideshow_init
-		};
-
-		type = g_type_register_static (GTK_TYPE_WINDOW,
-					       "GthSlideshow",
-					       &type_info,
-					       0);
-	}
-
-	return type;
-}
-
-
-static gboolean
-hide_cursor_cb (gpointer data)
-{
-	GthSlideshow *self = data;
-
-	g_source_remove (self->priv->hide_cursor_event);
-	self->priv->hide_cursor_event = 0;
-
 	clutter_stage_hide_cursor (CLUTTER_STAGE (self->stage));
-
-	return FALSE;
 }
 
 
 static void
-_gth_slideshow_toggle_pause (GthSlideshow *self)
+clutter_projector_paused (GthSlideshow *self)
 {
-	self->priv->paused = ! self->priv->paused;
-	if (self->priv->paused) {
-		if (self->priv->animating) {
-			clutter_timeline_pause (self->priv->timeline);
-			_gth_slideshow_animation_completed (self);
-		}
-#if HAVE_GSTREAMER
-		if (self->priv->playbin != NULL)
-			gst_element_set_state (self->priv->playbin, GST_STATE_PAUSED);
-#endif
-	}
-	else { /* resume */
-		_gth_slideshow_load_next_image (self);
-#if HAVE_GSTREAMER
-		if (self->priv->playbin != NULL)
-			gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
-#endif
+	if (self->priv->animating) {
+		clutter_timeline_pause (self->priv->timeline);
+		_gth_slideshow_animation_completed (self);
 	}
 }
 
@@ -537,6 +868,18 @@ stage_input_cb (ClutterStage *stage,
 			g_source_remove (self->priv->hide_cursor_event);
 		self->priv->hide_cursor_event = g_timeout_add (HIDE_CURSOR_DELAY, hide_cursor_cb, self);
 	}
+	else if (event->type == CLUTTER_BUTTON_PRESS) {
+		switch (clutter_event_get_button (event)) {
+		case 1:
+			_gth_slideshow_load_next_image (self);
+			break;
+		case 3:
+			_gth_slideshow_load_prev_image (self);
+			break;
+		default:
+			break;
+		}
+	}
 	else if (event->type == CLUTTER_KEY_RELEASE) {
 		switch (clutter_event_get_key_symbol (event)) {
 		case CLUTTER_Escape:
@@ -561,78 +904,86 @@ stage_input_cb (ClutterStage *stage,
 }
 
 
-#if HAVE_GSTREAMER
 static void
-bus_message_cb (GstBus     *bus,
-                GstMessage *message,
-                gpointer    user_data)
+gth_slideshow_size_allocate_cb (GtkWidget     *widget,
+				GtkAllocation *allocation,
+				gpointer       user_data)
 {
-	GthSlideshow *self = user_data;
+	GthSlideshow   *self = user_data;
+	gfloat          stage_w, stage_h;
+	GthImageLoader *image_loader;
+	GdkPixbuf      *pixbuf;
+	GdkPixbuf      *image;
+	int             pixbuf_w, pixbuf_h;
+	int             pixbuf_x, pixbuf_y;
+	ClutterActor   *texture;
 
-	if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_EOS) {
-		self->priv->current_audio_file++;
-		if ((self->priv->audio_files[self->priv->current_audio_file] == NULL)
-		    && self->priv->audio_loop)
-		{
-			self->priv->current_audio_file = 0;
-		}
-		gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
-		g_object_set (G_OBJECT (self->priv->playbin), "uri", self->priv->audio_files[self->priv->current_audio_file], NULL);
-		gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
-	}
-}
-#endif
-
-
-static void
-gth_slideshow_show_cb (GtkWidget    *widget,
-		       GthSlideshow *self)
-{
-	if (! self->priv->first_show)
+	if (self->current_image == NULL)
 		return;
 
-#if HAVE_GSTREAMER
-	if (gstreamer_init()) {
-		if ((self->priv->audio_files != NULL) && (self->priv->audio_files[0] != NULL)) {
-			self->priv->current_audio_file = 0;
-			if (self->priv->playbin == NULL) {
-				GstBus *bus;
+	clutter_actor_get_size (self->stage, &stage_w, &stage_h);
+	if ((stage_w == 0) || (stage_h == 0))
+		return;
 
-				self->priv->playbin = gst_element_factory_make ("playbin", "playbin");
-				bus = gst_pipeline_get_bus (GST_PIPELINE (self->priv->playbin));
-				gst_bus_add_signal_watch (bus);
-				g_signal_connect (bus, "message", G_CALLBACK (bus_message_cb), self);
-			}
-			else
-				gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
-			g_object_set (G_OBJECT (self->priv->playbin), "uri", self->priv->audio_files[self->priv->current_audio_file], NULL);
-			gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
-		}
-	}
-#endif
+	image_loader = gth_image_preloader_get_loader (self->priv->preloader, (GthFileData *) self->priv->current->data);
+	if (image_loader == NULL)
+		return;
 
-	_gth_slideshow_load_current_image (self);
-	self->priv->first_show = FALSE;
+	pixbuf = gth_image_loader_get_pixbuf (image_loader);
+	if (pixbuf == NULL)
+		return;
+
+	image = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (pixbuf),
+				FALSE,
+				gdk_pixbuf_get_bits_per_sample (pixbuf),
+				stage_w,
+				stage_h);
+	gdk_pixbuf_fill (image, 0x000000ff);
+
+	pixbuf_w = gdk_pixbuf_get_width (pixbuf);
+	pixbuf_h = gdk_pixbuf_get_height (pixbuf);
+	scale_keeping_ratio (&pixbuf_w, &pixbuf_h, (int) stage_w, (int) stage_h, TRUE);
+	pixbuf_x = (stage_w - pixbuf_w) / 2;
+	pixbuf_y = (stage_h - pixbuf_h) / 2;
+
+	gdk_pixbuf_composite (pixbuf,
+			      image,
+			      pixbuf_x,
+			      pixbuf_y,
+			      pixbuf_w,
+			      pixbuf_h,
+			      pixbuf_x,
+			      pixbuf_y,
+			      (double) pixbuf_w / gdk_pixbuf_get_width (pixbuf),
+			      (double) pixbuf_h / gdk_pixbuf_get_height (pixbuf),
+			      GDK_INTERP_BILINEAR,
+			      255);
+
+	if (self->current_image == self->priv->image1)
+		texture = self->priv->image1;
+	else
+		texture = self->priv->image2;
+	gtk_clutter_texture_set_from_pixbuf (CLUTTER_TEXTURE (texture), image, NULL);
+
+	self->current_geometry.x = 0;
+	self->current_geometry.y = 0;
+	self->current_geometry.width = stage_w;
+	self->current_geometry.height = stage_h;
+	_gth_slideshow_reset_textures_position (self);
 }
 
 
 static void
-_gth_slideshow_construct (GthSlideshow *self,
-			  GthBrowser   *browser,
-			  GList        *file_list)
+clutter_projector_construct (GthSlideshow *self)
 {
 	GtkWidget    *embed;
-	ClutterColor  stage_color = { 0x0, 0x0, 0x0, 0xff };
-
-	self->priv->browser = _g_object_ref (browser);
-	self->priv->file_list = _g_object_list_ref (file_list);
-	self->priv->current = self->priv->file_list;
-	self->priv->one_loaded = FALSE;
+	ClutterColor  background_color = { 0x0, 0x0, 0x0, 0xff };
 
 	embed = gtk_clutter_embed_new ();
 	self->stage = gtk_clutter_embed_get_stage (GTK_CLUTTER_EMBED (embed));
 	clutter_stage_hide_cursor (CLUTTER_STAGE (self->stage));
-	clutter_stage_set_color (CLUTTER_STAGE (self->stage), &stage_color);
+	clutter_stage_set_color (CLUTTER_STAGE (self->stage), &background_color);
+	g_signal_connect (self->stage, "button-press-event", G_CALLBACK (stage_input_cb), self);
 	g_signal_connect (self->stage, "motion-event", G_CALLBACK (stage_input_cb), self);
 	g_signal_connect (self->stage, "key-release-event", G_CALLBACK (stage_input_cb), self);
 
@@ -655,61 +1006,19 @@ _gth_slideshow_construct (GthSlideshow *self,
 	g_signal_connect (self->priv->timeline, "new-frame", G_CALLBACK (animation_frame_cb), self);
 	g_signal_connect (self->priv->timeline, "started", G_CALLBACK (animation_started_cb), self);
 
-	g_signal_connect (self, "show", G_CALLBACK (gth_slideshow_show_cb), self);
+	g_signal_connect (self, "size-allocate", G_CALLBACK (gth_slideshow_size_allocate_cb), self);
 }
 
 
-GtkWidget *
-gth_slideshow_new (GthBrowser *browser,
-		   GList      *file_list /* GthFileData */)
-{
-	GthSlideshow *window;
-
-	window = (GthSlideshow *) g_object_new (GTH_TYPE_SLIDESHOW, NULL);
-	_gth_slideshow_construct (window, browser, file_list);
-
-	return (GtkWidget*) window;
-}
+GthProjector clutter_projector = {
+	clutter_projector_construct,
+	clutter_projector_paused,
+	clutter_projector_hide_cursor,
+	clutter_projector_finalize,
+	clutter_projector_image_ready,
+	clutter_projector_load_prev_image,
+	clutter_projector_load_next_image
+};
 
 
-void
-gth_slideshow_set_delay (GthSlideshow *self,
-			 guint         msecs)
-{
-	self->priv->delay = msecs;
-}
-
-
-void
-gth_slideshow_set_automatic (GthSlideshow *self,
-			     gboolean      automatic)
-{
-	self->priv->automatic = automatic;
-}
-
-
-void
-gth_slideshow_set_wrap_around (GthSlideshow *self,
-			       gboolean      wrap_around)
-{
-	self->priv->wrap_around = wrap_around;
-}
-
-
-void
-gth_slideshow_set_transitions (GthSlideshow *self,
-			       GList        *transitions)
-{
-	_g_object_list_unref (self->priv->transitions);
-	self->priv->transitions = _g_object_list_ref (transitions);
-	self->priv->n_transitions = g_list_length (self->priv->transitions);
-}
-
-
-void
-gth_slideshow_set_playlist (GthSlideshow  *self,
-			    char         **files)
-{
-	self->priv->audio_files = g_strdupv (files);
-	self->priv->audio_loop = TRUE;
-}
+#endif /* HAVE_CLUTTER*/
