@@ -74,11 +74,23 @@ enum {
 
 struct _GthFolderTreePrivate
 {
-	GFile           *root;
-	GtkTreeStore    *tree_store;
-	GthIconCache    *icon_cache;
-	GtkCellRenderer *text_renderer;
-	GtkTreePath     *hover_path;
+	GFile            *root;
+	GtkTreeStore     *tree_store;
+	GthIconCache     *icon_cache;
+	GtkCellRenderer  *text_renderer;
+	GtkTreePath      *hover_path;
+
+	/* drag-and-drop */
+
+	gboolean         drag_source_enabled;
+	GdkModifierType  drag_start_button_mask;
+	GtkTargetList   *drag_target_list;
+	GdkDragAction    drag_actions;
+
+	gboolean         dragging : 1;        /* Whether the user is dragging items. */
+	gboolean         drag_started : 1;    /* Whether the drag has started. */
+	int              drag_start_x;        /* The position where the drag started. */
+	int              drag_start_y;
 };
 
 
@@ -94,6 +106,10 @@ gth_folder_tree_finalize (GObject *object)
 	folder_tree = GTH_FOLDER_TREE (object);
 
 	if (folder_tree->priv != NULL) {
+		if (folder_tree->priv->drag_target_list != NULL) {
+			gtk_target_list_unref (folder_tree->priv->drag_target_list);
+			folder_tree->priv->drag_target_list = NULL;
+		}
 		if (folder_tree->priv->root != NULL)
 			g_object_unref (folder_tree->priv->root);
 		gth_icon_cache_free (folder_tree->priv->icon_cache);
@@ -108,13 +124,17 @@ gth_folder_tree_finalize (GObject *object)
 static void
 gth_folder_tree_class_init (GthFolderTreeClass *class)
 {
-	GObjectClass *object_class;
+	GObjectClass   *object_class;
+	GtkWidgetClass *widget_class;
 
 	parent_class = g_type_class_peek_parent (class);
-	object_class = (GObjectClass*) class;
 
+	object_class = (GObjectClass*) class;
 	object_class->finalize = gth_folder_tree_finalize;
 
+	widget_class = (GtkWidgetClass*) class;
+	widget_class->drag_begin = NULL;
+
 	gth_folder_tree_signals[FOLDER_POPUP] =
 		g_signal_new ("folder_popup",
 			      G_TYPE_FROM_CLASS (class),
@@ -357,13 +377,6 @@ button_press_cb (GtkWidget      *widget,
 					     &cell_y))
 	{
 		if (event->button == 3) {
-			GtkTreeSelection *selection;
-
-			/* Update the selection. */
-
-			selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (folder_tree));
-			gtk_tree_selection_unselect_all (selection);
-
 			g_signal_emit (folder_tree,
 				       gth_folder_tree_signals[FOLDER_POPUP],
 				       0,
@@ -394,18 +407,6 @@ button_press_cb (GtkWidget      *widget,
 				    -1);
 
 		if (entry_type == ENTRY_TYPE_FILE) {
-			GtkTreeSelection *selection;
-
-			/* Update the selection. */
-
-			selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (folder_tree));
-			if (! gtk_tree_selection_iter_is_selected (selection, &iter)) {
-				gtk_tree_selection_unselect_all (selection);
-				gtk_tree_selection_select_iter (selection, &iter);
-			}
-
-			/* Show the folder popup menu. */
-
 			g_signal_emit (folder_tree,
 				       gth_folder_tree_signals[FOLDER_POPUP],
 				       0,
@@ -424,6 +425,19 @@ button_press_cb (GtkWidget      *widget,
 		int               expander_size;
 		int               horizontal_separator;
 
+		/* This can be the start of a dragging action. */
+
+		if (! (event->state & GDK_CONTROL_MASK)
+		    && ! (event->state & GDK_SHIFT_MASK)
+		    && folder_tree->priv->drag_source_enabled)
+		{
+			folder_tree->priv->dragging = TRUE;
+			folder_tree->priv->drag_start_x = event->x;
+			folder_tree->priv->drag_start_y = event->y;
+		}
+
+		/**/
+
 		if (! gtk_tree_view_column_cell_get_position (column,
 							      folder_tree->priv->text_renderer,
 							      &start_pos,
@@ -466,6 +480,87 @@ button_press_cb (GtkWidget      *widget,
 }
 
 
+static gboolean
+motion_notify_event_cb (GtkWidget      *widget,
+			GdkEventButton *event,
+			gpointer        user_data)
+{
+	GthFolderTree *folder_tree = user_data;
+
+	if (! folder_tree->priv->drag_source_enabled)
+		return FALSE;
+
+	if (folder_tree->priv->dragging) {
+		if (! folder_tree->priv->drag_started
+		    && gtk_drag_check_threshold (widget,
+					         folder_tree->priv->drag_start_x,
+					         folder_tree->priv->drag_start_y,
+						 event->x,
+						 event->y))
+		{
+			GtkTreePath    *path = NULL;
+			GdkDragContext *context;
+			int             cell_x;
+			int             cell_y;
+			GdkPixmap      *dnd_icon;
+
+			if (! gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (folder_tree),
+							     event->x,
+							     event->y,
+							     &path,
+							     NULL,
+							     &cell_x,
+							     &cell_y))
+			{
+				return FALSE;
+			}
+
+			gtk_tree_view_set_cursor (GTK_TREE_VIEW (folder_tree), path, NULL, FALSE);
+			folder_tree->priv->drag_started = TRUE;
+
+			/**/
+
+			context = gtk_drag_begin (widget,
+						  folder_tree->priv->drag_target_list,
+						  folder_tree->priv->drag_actions,
+						  1,
+						  (GdkEvent *) event);
+
+			dnd_icon = gtk_tree_view_create_row_drag_icon (GTK_TREE_VIEW (folder_tree), path);
+			gtk_drag_set_icon_pixmap (context,
+						  gdk_drawable_get_colormap (dnd_icon),
+						  dnd_icon,
+						  NULL,
+						  cell_x,
+						  cell_y);
+
+			g_object_unref (dnd_icon);
+			gtk_tree_path_free (path);
+		}
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+
+static gboolean
+button_release_event_cb (GtkWidget      *widget,
+			 GdkEventButton *event,
+			 gpointer        user_data)
+{
+	GthFolderTree *folder_tree = user_data;
+
+	if (folder_tree->priv->dragging) {
+		folder_tree->priv->dragging = FALSE;
+		folder_tree->priv->drag_started = FALSE;
+	}
+
+	return FALSE;
+}
+
+
 static void
 load_uri (GthFolderTree *folder_tree,
 	  EntryType      entry_type,
@@ -554,6 +649,64 @@ column_name_compare_func (GtkTreeModel *model,
 }
 
 
+static gboolean
+iter_stores_file (GtkTreeModel *tree_model,
+	          GtkTreeIter  *iter,
+	          GFile        *file,
+	          GtkTreeIter  *file_iter)
+{
+	GthFileData *iter_file_data;
+	EntryType    iter_type;
+	gboolean     found;
+
+	gtk_tree_model_get (tree_model, iter,
+			    COLUMN_FILE_DATA, &iter_file_data,
+			    COLUMN_TYPE, &iter_type,
+			    -1);
+	found = (iter_type == ENTRY_TYPE_FILE) && (iter_file_data != NULL) && g_file_equal (file, iter_file_data->file);
+
+	_g_object_unref (iter_file_data);
+
+	if (found)
+		*file_iter = *iter;
+
+	return found;
+}
+
+
+static gboolean
+_gth_folder_tree_find_file_in_children (GtkTreeModel *tree_model,
+				        GFile        *file,
+				        GtkTreeIter  *file_iter,
+				        GtkTreeIter  *root)
+{
+	GtkTreeIter iter;
+
+	/* check the children... */
+
+	if (! gtk_tree_model_iter_children (tree_model, &iter, root))
+		return FALSE;
+
+	do {
+		if (iter_stores_file (tree_model, &iter, file, file_iter))
+			return TRUE;
+	}
+	while (gtk_tree_model_iter_next (tree_model, &iter));
+
+	/* ...if no child stores the file, search recursively */
+
+	if (gtk_tree_model_iter_children (tree_model, &iter, root)) {
+		do {
+			if (_gth_folder_tree_find_file_in_children (tree_model, file, file_iter, &iter))
+				return TRUE;
+		}
+		while (gtk_tree_model_iter_next (tree_model, &iter));
+	}
+
+	return FALSE;
+}
+
+
 static gboolean
 gth_folder_tree_get_iter (GthFolderTree *folder_tree,
 			  GFile         *file,
@@ -561,36 +714,53 @@ gth_folder_tree_get_iter (GthFolderTree *folder_tree,
 			  GtkTreeIter   *root)
 {
 	GtkTreeModel *tree_model = GTK_TREE_MODEL (folder_tree->priv->tree_store);
-	GtkTreeIter   iter;
 
 	if (file == NULL)
 		return FALSE;
 
-	if (root != NULL) {
-		GthFileData *root_file_data;
-		EntryType    root_type;
-		gboolean     found;
+	if ((root != NULL) && iter_stores_file (tree_model, root, file, file_iter))
+		return TRUE;
 
-		gtk_tree_model_get (tree_model, root,
-				    COLUMN_FILE_DATA, &root_file_data,
-				    COLUMN_TYPE, &root_type,
-				    -1);
-		found = (root_type == ENTRY_TYPE_FILE) && (root_file_data != NULL) && g_file_equal (file, root_file_data->file);
+	/* This type of search is useful to give priority to the first level
+	 * of entries which contains all the entry points.
+	 * For example if file is "file:///media/usb-disk" this function must
+	 * return the entry point corresponding to the device instead of
+	 * returing the usb-disk folder located in "file:///media". */
 
-		_g_object_unref (root_file_data);
+	if (_gth_folder_tree_find_file_in_children (tree_model, file, file_iter, root))
+		return TRUE;
 
-		if (found) {
-			*file_iter = *root;
-			return TRUE;
-		}
-	}
+	return FALSE;
+}
 
-	if (! gtk_tree_model_iter_children (tree_model, &iter, root))
+
+static gboolean
+_gth_folder_tree_get_child (GthFolderTree *folder_tree,
+			    GFile         *file,
+			    GtkTreeIter   *file_iter,
+			    GtkTreeIter   *parent)
+{
+	GtkTreeModel *tree_model = GTK_TREE_MODEL (folder_tree->priv->tree_store);
+	GtkTreeIter   iter;
+
+	if (! gtk_tree_model_iter_children (tree_model, &iter, parent))
 		return FALSE;
 
 	do {
-		if (gth_folder_tree_get_iter (folder_tree, file, file_iter, &iter))
+		GthFileData *test_file_data;
+		EntryType    file_entry_type;
+
+		gtk_tree_model_get (tree_model, &iter,
+				    COLUMN_FILE_DATA, &test_file_data,
+				    COLUMN_TYPE, &file_entry_type,
+				    -1);
+		if ((file_entry_type == ENTRY_TYPE_FILE) && (test_file_data != NULL) && g_file_equal (file, test_file_data->file)) {
+			_g_object_unref (test_file_data);
+			*file_iter = iter;
 			return TRUE;
+		}
+
+		_g_object_unref (test_file_data);
 	}
 	while (gtk_tree_model_iter_next (tree_model, &iter));
 
@@ -626,12 +796,13 @@ _gth_folder_tree_child_type_present (GthFolderTree *folder_tree,
 
 static void
 _gth_folder_tree_add_loading_item (GthFolderTree *folder_tree,
-				   GtkTreeIter   *parent)
+				   GtkTreeIter   *parent,
+				   gboolean       forced)
 {
 	char        *sort_key;
 	GtkTreeIter  iter;
 
-	if (_gth_folder_tree_child_type_present (folder_tree, parent, ENTRY_TYPE_LOADING))
+	if (! forced && _gth_folder_tree_child_type_present (folder_tree, parent, ENTRY_TYPE_LOADING))
 		return;
 
 	sort_key = g_utf8_collate_key_for_filename (LOADING_URI, -1);
@@ -651,12 +822,13 @@ _gth_folder_tree_add_loading_item (GthFolderTree *folder_tree,
 
 static void
 _gth_folder_tree_add_empty_item (GthFolderTree *folder_tree,
-				 GtkTreeIter   *parent)
+				 GtkTreeIter   *parent,
+				 gboolean       forced)
 {
 	char        *sort_key;
 	GtkTreeIter  iter;
 
-	if (_gth_folder_tree_child_type_present (folder_tree, parent, ENTRY_TYPE_EMPTY))
+	if (! forced && _gth_folder_tree_child_type_present (folder_tree, parent, ENTRY_TYPE_EMPTY))
 		return;
 
 	sort_key = g_utf8_collate_key_for_filename (EMPTY_URI, -1);
@@ -674,20 +846,22 @@ _gth_folder_tree_add_empty_item (GthFolderTree *folder_tree,
 }
 
 
-static void
+static gboolean
 _gth_folder_tree_set_file_data (GthFolderTree *folder_tree,
 				GtkTreeIter   *iter,
 				GthFileData   *file_data)
 {
-	GIcon      *icon;
-	GdkPixbuf  *pixbuf;
 	const char *name;
 	char       *sort_key;
+	GIcon      *icon;
+	GdkPixbuf  *pixbuf;
 
+	name = g_file_info_get_display_name (file_data->info);
+	if (name == NULL)
+		return FALSE;
+	sort_key = g_utf8_collate_key_for_filename (name, -1);
 	icon = g_file_info_get_icon (file_data->info);
 	pixbuf = gth_icon_cache_get_pixbuf (folder_tree->priv->icon_cache, icon);
-	name = g_file_info_get_display_name (file_data->info);
-	sort_key = g_utf8_collate_key_for_filename (name, -1);
 
 	gtk_tree_store_set (folder_tree->priv->tree_store, iter,
 			    COLUMN_STYLE, PANGO_STYLE_NORMAL,
@@ -703,6 +877,8 @@ _gth_folder_tree_set_file_data (GthFolderTree *folder_tree,
 
 	g_free (sort_key);
 	_g_object_unref (pixbuf);
+
+	return TRUE;
 }
 
 
@@ -736,7 +912,10 @@ _gth_folder_tree_add_file (GthFolderTree *folder_tree,
 	/* add the folder */
 
 	gtk_tree_store_append (folder_tree->priv->tree_store, &iter, parent);
-	_gth_folder_tree_set_file_data (folder_tree, &iter, fd);
+	if (! _gth_folder_tree_set_file_data (folder_tree, &iter, fd)) {
+		gtk_tree_store_remove (folder_tree->priv->tree_store, &iter);
+		return FALSE;
+	}
 
 	if (g_file_info_get_attribute_boolean (fd->info, "gthumb::entry-point"))
 		gtk_tree_store_set (folder_tree->priv->tree_store, &iter,
@@ -747,8 +926,8 @@ _gth_folder_tree_add_file (GthFolderTree *folder_tree,
 				    COLUMN_WEIGHT, PANGO_WEIGHT_NORMAL,
 				    -1);
 
-	if (! _gth_folder_tree_iter_has_no_child (folder_tree, &iter))
-		_gth_folder_tree_add_loading_item (folder_tree, &iter);
+	if (! g_file_info_get_attribute_boolean (fd->info, "gthumb::no-child"))
+		_gth_folder_tree_add_loading_item (folder_tree, &iter, TRUE);
 
 	return TRUE;
 }
@@ -793,15 +972,23 @@ gth_folder_tree_construct (GthFolderTree *folder_tree)
 
 	/**/
 
-	g_signal_connect (G_OBJECT (folder_tree),
-			  "button_press_event",
+	g_signal_connect (folder_tree,
+			  "button-press-event",
 			  G_CALLBACK (button_press_cb),
 			  folder_tree);
-	g_signal_connect (G_OBJECT (folder_tree),
+	g_signal_connect (folder_tree,
+			  "motion-notify-event",
+			  G_CALLBACK (motion_notify_event_cb),
+			  folder_tree);
+	g_signal_connect (folder_tree,
+			  "button-release-event",
+			  G_CALLBACK (button_release_event_cb),
+			  folder_tree);
+	g_signal_connect (folder_tree,
 			  "row-activated",
 			  G_CALLBACK (row_activated_cb),
 			  folder_tree);
-	g_signal_connect (G_OBJECT (folder_tree),
+	g_signal_connect (folder_tree,
 			  "row-expanded",
 			  G_CALLBACK (row_expanded_cb),
 			  folder_tree);
@@ -812,6 +999,10 @@ static void
 gth_folder_tree_init (GthFolderTree *folder_tree)
 {
 	folder_tree->priv = g_new0 (GthFolderTreePrivate, 1);
+	folder_tree->priv->drag_source_enabled = FALSE;
+	folder_tree->priv->dragging = FALSE;
+	folder_tree->priv->drag_started = FALSE;
+	folder_tree->priv->drag_target_list = NULL;
 	gth_folder_tree_construct (folder_tree);
 }
 
@@ -930,6 +1121,7 @@ emit_fake_motion_notify_event (GthFolderTree *folder_tree)
 }
 
 
+G_GNUC_UNUSED
 static GList *
 _gth_folder_tree_get_children (GthFolderTree *folder_tree,
 			       GtkTreeIter   *parent)
@@ -987,15 +1179,17 @@ _gth_folder_tree_remove_child_type (GthFolderTree *folder_tree,
 
 void
 gth_folder_tree_set_children (GthFolderTree *folder_tree,
-			      GFile         *parent,
-			      GList         *files)
+			       GFile         *parent,
+			       GList         *files)
 {
-	GtkTreeIter  parent_iter;
-	GtkTreeIter *p_parent_iter;
-	gboolean     is_empty;
-	GList       *old_files;
-	GList       *scan;
-	GtkTreeIter  iter;
+	GtkTreeIter   parent_iter;
+	GtkTreeIter  *p_parent_iter;
+	gboolean      is_empty;
+	GHashTable   *file_hash;
+	GList        *scan;
+	GList        *old_files;
+	GtkTreeModel *tree_model;
+	GtkTreeIter   iter;
 
 	if (g_file_equal (parent, folder_tree->priv->root))
 		p_parent_iter = NULL;
@@ -1007,37 +1201,84 @@ gth_folder_tree_set_children (GthFolderTree *folder_tree,
 	if (_gth_folder_tree_iter_has_no_child (folder_tree, p_parent_iter))
 		return;
 
+	tree_model = GTK_TREE_MODEL (folder_tree->priv->tree_store);
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (tree_model), GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, 0);
+
 	is_empty = TRUE;
-	_gth_folder_tree_add_empty_item (folder_tree, p_parent_iter);
-	_gth_folder_tree_remove_child_type (folder_tree, p_parent_iter, ENTRY_TYPE_LOADING);
+	_gth_folder_tree_add_empty_item (folder_tree, p_parent_iter, FALSE);
 
-	/* delete the children not present in the new file list */
+	/* delete the children not present in the new file list, update the
+	 * already existing files */
 
-	old_files = _gth_folder_tree_get_children (folder_tree, p_parent_iter);
+	file_hash = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, g_object_unref, NULL);
+	for (scan = files; scan; scan = scan->next) {
+		GthFileData *file_data = scan->data;
+		g_hash_table_insert (file_hash, g_object_ref (file_data->file), GINT_TO_POINTER (1));
+	}
+
+	old_files = NULL;
+	if (gtk_tree_model_iter_children (tree_model, &iter, p_parent_iter)) {
+		gboolean valid = TRUE;
+
+		do {
+			GthFileData *file_data = NULL;
+			EntryType    file_type;
+
+			gtk_tree_model_get (tree_model, &iter,
+					    COLUMN_FILE_DATA, &file_data,
+					    COLUMN_TYPE, &file_type,
+					    -1);
+
+			if (file_type == ENTRY_TYPE_LOADING) {
+				valid = gtk_tree_store_remove (folder_tree->priv->tree_store, &iter);
+			}
+			else if (file_type == ENTRY_TYPE_FILE) {
+				/* save the old files list to compute the new files list below */
+				old_files = g_list_prepend (old_files, g_object_ref (file_data));
+
+				if (g_hash_table_lookup (file_hash, file_data->file)) {
+					/* file_data is already present in the list, just update it */
+					if (_gth_folder_tree_set_file_data (folder_tree, &iter, file_data))
+						is_empty = FALSE;
+					valid = gtk_tree_model_iter_next (tree_model, &iter);
+				}
+				else {
+					/* file_data is not present anymore, remove it from the tree */
+					valid = gtk_tree_store_remove (folder_tree->priv->tree_store, &iter);
+				}
+			}
+			else
+				valid = gtk_tree_model_iter_next (tree_model, &iter);
+
+			_g_object_unref (file_data);
+		}
+		while (valid);
+	}
+
+	g_hash_table_unref (file_hash);
+
+	/* add the new files */
+
+	file_hash = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, g_object_unref, NULL);
 	for (scan = old_files; scan; scan = scan->next) {
 		GthFileData *file_data = scan->data;
-
-		if (! gth_file_data_list_find_file (files, file_data->file)
-		    && gth_folder_tree_get_iter (folder_tree, file_data->file, &iter, p_parent_iter))
-		{
-			gtk_tree_store_remove (folder_tree->priv->tree_store, &iter);
-		}
+		g_hash_table_insert (file_hash, g_object_ref (file_data->file), GINT_TO_POINTER (1));
 	}
-	_g_object_list_unref (old_files);
-
-	/* add or update the new files */
 
 	for (scan = files; scan; scan = scan->next) {
 		GthFileData *file_data = scan->data;
 
-		if (gth_folder_tree_get_iter (folder_tree, file_data->file, &iter, p_parent_iter)) {
-			_gth_folder_tree_set_file_data (folder_tree, &iter, file_data);
-			is_empty = FALSE;
+		if (! g_hash_table_lookup (file_hash, file_data->file)) {
+			if (_gth_folder_tree_add_file (folder_tree, p_parent_iter, file_data))
+				is_empty = FALSE;
 		}
-		else if (_gth_folder_tree_add_file (folder_tree, p_parent_iter, file_data))
-			is_empty = FALSE;
 	}
 
+	_g_object_list_unref (old_files);
+	g_hash_table_unref (file_hash);
+
+	/**/
+
 	if (! is_empty)
 		_gth_folder_tree_remove_child_type (folder_tree, p_parent_iter, ENTRY_TYPE_EMPTY);
 
@@ -1046,6 +1287,8 @@ gth_folder_tree_set_children (GthFolderTree *folder_tree,
 				    COLUMN_LOADED, TRUE,
 				    -1);
 
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (folder_tree->priv->tree_store), COLUMN_NAME, GTK_SORT_ASCENDING);
+
 	emit_fake_motion_notify_event (folder_tree);
 }
 
@@ -1066,7 +1309,7 @@ gth_folder_tree_loading_children (GthFolderTree *folder_tree,
 	else
 		return;
 
-	_gth_folder_tree_add_loading_item (folder_tree, p_parent_iter);
+	_gth_folder_tree_add_loading_item (folder_tree, p_parent_iter, FALSE);
 
 	/* remove anything but the loading item */
 	gtk_tree_model_iter_children (GTK_TREE_MODEL (folder_tree->priv->tree_store), &iter, p_parent_iter);
@@ -1150,40 +1393,6 @@ gth_folder_tree_add_children (GthFolderTree *folder_tree,
 }
 
 
-static gboolean
-_gth_folder_tree_get_child (GthFolderTree *folder_tree,
-			    GFile         *file,
-			    GtkTreeIter   *file_iter,
-			    GtkTreeIter   *parent)
-{
-	GtkTreeModel *tree_model = GTK_TREE_MODEL (folder_tree->priv->tree_store);
-	GtkTreeIter   iter;
-
-	if (! gtk_tree_model_iter_children (tree_model, &iter, parent))
-		return FALSE;
-
-	do {
-		GthFileData *test_file_data;
-		EntryType    file_entry_type;
-
-		gtk_tree_model_get (tree_model, &iter,
-				    COLUMN_FILE_DATA, &test_file_data,
-				    COLUMN_TYPE, &file_entry_type,
-				    -1);
-		if ((file_entry_type == ENTRY_TYPE_FILE) && (test_file_data != NULL) && g_file_equal (file, test_file_data->file)) {
-			_g_object_unref (test_file_data);
-			*file_iter = iter;
-			return TRUE;
-		}
-
-		_g_object_unref (test_file_data);
-	}
-	while (gtk_tree_model_iter_next (tree_model, &iter));
-
-	return FALSE;
-}
-
-
 void
 gth_folder_tree_update_children (GthFolderTree *folder_tree,
 				 GFile         *parent,
@@ -1244,13 +1453,13 @@ gth_folder_tree_delete_children (GthFolderTree *folder_tree,
 		return;
 
 	/* add the empty item first to not allow the folder to collapse. */
-	_gth_folder_tree_add_empty_item (folder_tree, p_parent_iter);
+	_gth_folder_tree_add_empty_item (folder_tree, p_parent_iter, TRUE);
 
 	for (scan = files; scan; scan = scan->next) {
 		GFile       *file = scan->data;
 		GtkTreeIter  iter;
 
-		if (gth_folder_tree_get_iter (folder_tree, file, &iter, p_parent_iter))
+		if (_gth_folder_tree_get_child (folder_tree, file, &iter, p_parent_iter))
 			gtk_tree_store_remove (folder_tree->priv->tree_store, &iter);
 	}
 
@@ -1358,6 +1567,34 @@ gth_folder_tree_expand_row (GthFolderTree *folder_tree,
 }
 
 
+GthFileData *
+gth_folder_tree_get_file (GthFolderTree *folder_tree,
+			  GtkTreePath   *path)
+{
+	GtkTreeModel *tree_model;
+	GtkTreeIter   iter;
+	EntryType     entry_type;
+	GthFileData  *file_data;
+
+	tree_model = GTK_TREE_MODEL (folder_tree->priv->tree_store);
+	if (! gtk_tree_model_get_iter (tree_model, &iter, path))
+			return NULL;
+
+	file_data = NULL;
+	gtk_tree_model_get (tree_model,
+			    &iter,
+			    COLUMN_TYPE, &entry_type,
+			    COLUMN_FILE_DATA, &file_data,
+			    -1);
+	if (entry_type != ENTRY_TYPE_FILE) {
+		_g_object_unref (file_data);
+		file_data = NULL;
+	}
+
+	return file_data;
+}
+
+
 void
 gth_folder_tree_select_path (GthFolderTree *folder_tree,
 			     GtkTreePath   *path)
@@ -1448,3 +1685,28 @@ gth_folder_tree_get_selected_or_parent (GthFolderTree *folder_tree)
 
 	return file_data;
 }
+
+
+void
+gth_folder_tree_enable_drag_source (GthFolderTree        *self,
+				    GdkModifierType       start_button_mask,
+				    const GtkTargetEntry *targets,
+				    int                   n_targets,
+				    GdkDragAction         actions)
+{
+	if (self->priv->drag_target_list != NULL)
+		gtk_target_list_unref (self->priv->drag_target_list);
+
+	self->priv->drag_source_enabled = TRUE;
+	self->priv->drag_start_button_mask = start_button_mask;
+	self->priv->drag_target_list = gtk_target_list_new (targets, n_targets);
+	self->priv->drag_actions = actions;
+}
+
+
+void
+gth_folder_tree_unset_drag_source (GthFolderTree *self)
+{
+	self->priv->drag_source_enabled = FALSE;
+}
+
