@@ -34,6 +34,7 @@
 #define DRAG_THRESHOLD 1
 #define STEP_INCREMENT 20.0  /* scroll increment. */
 #define SCROLL_TIMEOUT 30    /* autoscroll timeout in milliseconds */
+#define GOLDEN_RATIO   1.6180339887
 
 
 typedef struct {
@@ -181,6 +182,7 @@ enum {
 	SELECTED,
 	MOTION_NOTIFY,
 	MASK_VISIBILITY_CHANGED,
+	GRID_VISIBILITY_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -200,6 +202,7 @@ struct _GthImageSelectorPrivate {
 	gboolean         use_ratio;
 	double           ratio;
 	gboolean         mask_visible;
+	GthGridType      grid_type;
 	gboolean         active;
 
 	GdkRectangle     drag_start_selection_area;
@@ -635,6 +638,49 @@ paint_selection (GthImageSelector *self,
 				paint_area.width,
 				paint_area.height,
 				GDK_INTERP_TILES);
+
+	if (self->priv->grid_type != GTH_GRID_NONE) {
+		int grid_x0, grid_x1, grid_x2, grid_x3;
+	        int grid_y0, grid_y1, grid_y2, grid_y3;
+		int x_delta, y_delta;
+
+		grid_x0 = paint_area.x;
+		grid_x3 = paint_area.x + paint_area.width;
+
+                grid_y0 = paint_area.y;
+                grid_y3 = paint_area.y + paint_area.height;
+
+		if (self->priv->grid_type == GTH_GRID_THIRDS) {
+			x_delta = paint_area.width / 3;
+			y_delta = paint_area.height /3;
+		}
+		else if (self->priv->grid_type == GTH_GRID_GOLDEN) {
+			x_delta = paint_area.width * (GOLDEN_RATIO / (1.0 + 2.0 * GOLDEN_RATIO));
+			y_delta = paint_area.height * (GOLDEN_RATIO / (1.0 + 2.0 * GOLDEN_RATIO));
+		}
+
+		grid_x1 = grid_x0 + x_delta;
+		grid_x2 = grid_x3 - x_delta;
+                grid_y1 = grid_y0 + y_delta;
+                grid_y2 = grid_y3 - y_delta;
+
+		gdk_draw_line (GTK_WIDGET (self->priv->viewer)->window,
+			       self->priv->selection_gc,
+			       grid_x1, grid_y0,
+			       grid_x1, grid_y3);
+	        gdk_draw_line (GTK_WIDGET (self->priv->viewer)->window,
+       		               self->priv->selection_gc,
+ 			       grid_x2, grid_y0,
+			       grid_x2, grid_y3);
+        	gdk_draw_line (GTK_WIDGET (self->priv->viewer)->window,
+       	        	       self->priv->selection_gc,
+                               grid_x0, grid_y1,
+                               grid_x3, grid_y1);
+	        gdk_draw_line (GTK_WIDGET (self->priv->viewer)->window,
+       		               self->priv->selection_gc,
+                               grid_x0, grid_y2,
+                               grid_x3, grid_y2);
+	}
 }
 
 
@@ -893,12 +939,12 @@ gth_image_selector_button_press (GthImageViewerTool *base,
 			set_active_area (self, get_event_area_from_id (self, C_BOTTOM_RIGHT_AREA));
 		}
 		else if (self->priv->type == GTH_SELECTOR_TYPE_POINT) {
-			retval = TRUE;
 			g_signal_emit (G_OBJECT (self),
 				       signals[SELECTED],
 				       0,
 				       new_selection.x,
 				       new_selection.y);
+			return TRUE;
 		}
 	}
 
@@ -1281,6 +1327,7 @@ gth_image_selector_instance_init (GthImageSelector *self)
 	self->priv->type = GTH_SELECTOR_TYPE_REGION;
 	self->priv->ratio = 1.0;
 	self->priv->mask_visible = TRUE;
+	self->priv->grid_type = GTH_GRID_NONE;
 }
 
 
@@ -1349,6 +1396,14 @@ gth_image_selector_class_init (GthImageSelectorClass *class)
 							 g_cclosure_marshal_VOID__VOID,
 							 G_TYPE_NONE,
 							 0);
+        signals[GRID_VISIBILITY_CHANGED] = g_signal_new ("grid_visibility_changed",
+                                                         G_TYPE_FROM_CLASS (class),
+                                                         G_SIGNAL_RUN_LAST,
+                                                         G_STRUCT_OFFSET (GthImageSelectorClass, grid_visibility_changed),
+                                                         NULL, NULL,
+                                                         g_cclosure_marshal_VOID__VOID,
+                                                         G_TYPE_NONE,
+                                                         0);
 }
 
 
@@ -1553,9 +1608,30 @@ gth_image_selector_set_mask_visible (GthImageSelector *self,
 }
 
 
+void
+gth_image_selector_set_grid_type (GthImageSelector *self,
+                                  GthGridType       grid_type)
+{
+        if (grid_type == self->priv->grid_type)
+                return;
+
+        self->priv->grid_type = grid_type;
+        gtk_widget_queue_draw (GTK_WIDGET (self->priv->viewer));
+        g_signal_emit (G_OBJECT (self),
+                       signals[GRID_VISIBILITY_CHANGED],
+                       0);
+}
+
+
 gboolean
 gth_image_selector_get_mask_visible (GthImageSelector *self)
 {
 	return self->priv->mask_visible;
 }
 
+
+GthGridType
+gth_image_selector_get_grid_type (GthImageSelector *self)
+{
+        return self->priv->grid_type;
+}
