@@ -172,7 +172,7 @@ gth_main_init (GthMain *main)
 							g_str_equal,
 							NULL,
 							NULL);
-	main->priv->loaders = g_hash_table_new (g_str_hash, g_str_equal);
+	main->priv->loaders = g_hash_table_new (g_str_hash, (GEqualFunc) g_content_type_equals);
 	main->priv->metadata_category = g_ptr_array_new ();
 	main->priv->metadata_info = g_ptr_array_new ();
 	main->priv->metadata_info_sorted = FALSE;
@@ -342,20 +342,10 @@ gth_main_get_all_entry_points (void)
 
 	for (scan = gth_main_get_all_file_sources (); scan; scan = scan->next) {
 		GthFileSource *file_source = scan->data;
-		GList         *entry_points;
-		GList         *scan_entry;
-
-		entry_points = gth_file_source_get_entry_points (file_source);
-		for (scan_entry = entry_points; scan_entry; scan_entry = scan_entry->next) {
-			GthFileData *file_data = scan_entry->data;
-
-			list = g_list_prepend (list, g_object_ref (file_data));
-		}
-
-		_g_object_list_unref (entry_points);
+		list = g_list_concat (list, gth_file_source_get_entry_points (file_source));
 	}
 
-	return g_list_reverse (list);
+	return list;
 }
 
 
@@ -417,6 +407,9 @@ gth_main_register_metadata_info (GthMetadataInfo *metadata_info)
 {
 	GthMetadataInfo *info;
 
+	if ((metadata_info->display_name != NULL) && (strstr (metadata_info->display_name, "0x") != NULL))
+		return NULL;
+
 	g_static_mutex_lock (&metadata_info_mutex);
 
 	info = gth_metadata_info_dup (metadata_info);
@@ -437,7 +430,8 @@ gth_main_register_metadata_info_v (GthMetadataInfo metadata_info[])
 	g_static_mutex_lock (&metadata_info_mutex);
 
 	for (i = 0; metadata_info[i].id != NULL; i++)
-		g_ptr_array_add (Main->priv->metadata_info, &metadata_info[i]);
+		if ((metadata_info[i].display_name == NULL) || (strstr (metadata_info[i].display_name, "0x") == NULL))
+			g_ptr_array_add (Main->priv->metadata_info, &metadata_info[i]);
 
 	g_static_mutex_unlock (&metadata_info_mutex);
 }
@@ -718,7 +712,7 @@ gth_main_register_pixbuf_loader (PixbufLoader  loader,
 	va_start (var_args, first_mime_type);
 	mime_type = first_mime_type;
   	while (mime_type != NULL) {
-		g_hash_table_insert (Main->priv->loaders, (gpointer) get_static_string (mime_type), loader);
+		g_hash_table_insert (Main->priv->loaders, (gpointer) get_static_string (g_content_type_from_mime_type (mime_type)), loader);
 		mime_type = va_arg (var_args, const char *);
   	}
 	va_end (var_args);
@@ -1197,6 +1191,7 @@ gth_main_activate_extensions (void)
 						"desktop_background",
 						"edit_metadata",
 						"exiv2_tools",
+						"facebook",
 						"file_manager",
 						"file_tools",
 						"flicker",
@@ -1208,7 +1203,9 @@ gth_main_activate_extensions (void)
 						"importer",
 						"jpeg_utils",
 						"list_tools",
+						"oauth",
 						"photo_importer",
+						"photobucket",
 						"picasaweb",
 						"pixbuf_savers",
 						"raw_files",
